/*
LIBRERIA PARA EL SENSOR ULTRASONICO
 * Esta librera ofrece la distancia en las siguies unidades
        * centimeotrs [cm]
        * metros [m]
        * pulgadas [in]
        * pies [ft]
 * El valor de la distancia debe ser almacenada en una varible de tipo float
            Por ejemplo: float x = DIST_cm();
 * El oscilador debe ser configurado a 48 MHz
 * Esta libreria hace uso del timer 1 para conocer la distancia 
 * Antes de llamar a una funcion para medir la distancia debe llamar a la funcion INICIAR

 * Calculo de la distancia
    v = d/t -------> d = v*t
    Tiempo_avance = 4/FOSC * Prescaler = 4/48000000 * 8 = 0.000666 ms
    d = (1/2) * (Tiempo_avance[ms] * timer) * velocidad del sonido
        * En centimetros
            distancia = (1/2) * (Tiempo_avance[ms] * timer) * 34 [cm/ms]
        * En metros
            distancia = (1/2) * (Tiempo_avance[ms] * timer) * 0.34 [m/ms]
        * En pulgadas
            distancia = (1/2) * (Tiempo_avance[ms] * timer) * 13.5 [in/ms]
        * En pies 
 */

//Declaracion de pines de la librera --- Cambiar al gusto si es necesario
#define ECHO        PORTBbits.RB7
#define TRIGGER     LATBbits.LB6

//Variables de la librera
unsigned int distancia = 0;

//Prototipos de funcion de la libreria
int DIST_cm();
int DIST_m();
int DIST_in();
int DIST_ft();

//Funcin para configurar el sensor
void ULTRA_Init(void){
    T1CON = 0b11111001; //TIMER1 es usado para conocer el tiempo
}

//Funcion para medir distancia en centimetros
int DIST_cm(){
    TRIGGER = 1;
    __delay_us(10);
    TRIGGER = 0;
    while(ECHO == 0){
    }
    //Reiniciar valores de los registos
    TMR1H = 0x00;
    TMR1L = 0x00;
    while(ECHO == 1){
    }
    distancia = ((TMR1H<<8) + TMR1L)/2.0*0.000666*34.0*10.0;
    /*La distancia la devuelve con un numero entero de cuatro digitos
     * (Esto facilita la separacin de dgitos)
    Pero debe entenderse que debe colocarse un punto decimal despues del tercer digito
    Ejemplo: 
        Valor alamcenado: 2435 
        Valor a mostrar en la LCD: 243.5 cm
    */
    return distancia;
}

//Funcion para medir distancia en metros
int DIST_m(){
    TRIGGER = 1;
    __delay_us(10);
    TRIGGER = 0;
    while(ECHO == 0){
    }
    //Reiniciar valores de los registos
    TMR1H = 0x00;
    TMR1L = 0x00;
    while(ECHO == 1){
    }
    distancia = ((TMR1H<<8) + TMR1L)/2.0*0.000666*0.34*1000.0;
    /*La distancia la devuelve con un numero entero de cuatro digitos
     * (Esto facilita la separacin de dgitos)
    Pero debe entenderse que debe colocarse un punto decimal despues del primer digito
    Ejemplo: 
        Valor alamcenado: 2435 
        Valor a mostrar en la LCD: 2.435 m
    */
    return distancia;
}
 
//Funcion para medir distancia en pulgadas
int DIST_in(){
    TRIGGER = 1;
    __delay_us(10);
    TRIGGER = 0;
    while(ECHO == 0){
    }
    //Reiniciar valores de los registos
    TMR1H = 0x00;
    TMR1L = 0x00;
    while(ECHO == 1){
    }
    distancia = ((TMR1H<<8) + TMR1L)/2.0*0.000666*13.5*10.0;
    /*La distancia la devuelve con un numero entero de cuatro digitos
     * (Esto facilita la separacin de dgitos)
    Pero debe entenderse que debe colocarse un punto decimal despues del primer digito
    Ejemplo: 
        Valor alamcenado: 0958 
        Valor a mostrar en la LCD: 95.8 in
    */
    return distancia;
    
}

//Funcion para medir distancia en pies
int DIST_ft(){
    TRIGGER = 1;
    __delay_us(10);
    TRIGGER = 0;
    while(ECHO == 0){
    }
    //Reiniciar valores de los registos
    TMR1H = 0x00;
    TMR1L = 0x00;
    while(ECHO == 1){
    }
    distancia = ((TMR1H<<8) + TMR1L)/2.0*0.000666*1.1*100.0;
    /*La distancia la devuelve con un numero entero de cuatro digitos
     * (Esto facilita la separacin de dgitos)
    Pero debe entenderse que debe colocarse un punto decimal despues del segundo digito
    Ejemplo: 
        Valor alamcenado: 0798 
        Valor a mostrar en la LCD: 7.98 ft
    */
    return distancia;
}